package com.xydl.cac.iec;

import com.beanit.iec61850bean.BasicDataAttribute;
import com.beanit.iec61850bean.FileInformation;
import com.xydl.cac.config.BizConfig;
import com.xydl.cac.entity.*;
import com.xydl.cac.entity.constants.Constants;
import com.xydl.cac.model.StaticVariable;
import com.xydl.cac.repository.*;
import com.xydl.cac.service.DataService;
import com.xydl.cac.service.IedDlRecordService;
import com.xydl.cac.socket.WebSocketServer;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Slf4j
public class IEDCollectService {
    IcdConfigTypeRepository _configRepository;
    IcdConfigTypeAttRepository _attRepository;
    IcdConfigTypeInstRepository _instRepository;
    RptparamindexRepository _rptparamindexRepository;
    IedDlRecordService _dlRecordService;
    DataService _dataService;
    IecClient iecClient;
    IcdIed ied;
    String xml;
    WebSocketServer _webSocketServer;
    BizConfig _bizConfig;

    String folder = "/record";

    public IEDCollectService(IcdConfigTypeRepository configRepository, IcdConfigTypeAttRepository attRepository,
                             IcdConfigTypeInstRepository instRepository, RptparamindexRepository rptparamindexRepository,
                             IedDlRecordService dlRecordService, DataService dataService,
                             String xml, IcdIed ied,
                             WebSocketServer webSocketServer, BizConfig bizConfig) {
        _configRepository = configRepository;
        _attRepository = attRepository;
        _instRepository = instRepository;
        _rptparamindexRepository = rptparamindexRepository;
        _dlRecordService = dlRecordService;
        _dataService = dataService;
        this.xml = xml;
        this.ied = ied;
        _webSocketServer = webSocketServer;
        _bizConfig = bizConfig;
        iecClient = new IecClient();
    }

    public void connect() throws Exception {
        iecClient.connect(ied, xml);
    }

    public void disconnect() {
        iecClient.disconnect();
    }

    public void collectAndSave(List<Rptparamindex> rptList, List<IedDlConfig> dlList) {
        List<IcdConfigType> configTypeList = _configRepository.findByIcdIedId(ied.getId());
        if (CollectionUtils.isEmpty(configTypeList)) {
            return;
        }
        // 开始连iec61850采集数据
        try {
            log.info("61850开始采集数据, ied=" + ied.getName() + ", ip=" + ied.getIp() + ", port=" + ied.getPort());
            this.connect();
            if (!CollectionUtils.isEmpty(rptList)) {
                this.doCollectAndSave(configTypeList, rptList);
            }

            if (!CollectionUtils.isEmpty(dlList)) {
                this.doDownload(dlList);
            }
        } catch (Exception ex) {
            String err = "61850采集数据异常, ied=" + ied.getName() + ", ip=" + ied.getIp() + ", port=" + ied.getPort();
            log.error(err, ex);
            String key = ied.getName() + ied.getIp() + ied.getPort();
            if (!StaticVariable.doneWarnMap.containsKey(key)) {
                StaticVariable.doneWarnMap.put(key, "1");
                _webSocketServer.sendMessage(err, null);
            }
            RealTimeDataService.onErrorCheck(ied.getId());
        } finally {
            iecClient.disconnect();
        }
    }

    private void doCollectAndSave(List<IcdConfigType> configTypeList, List<Rptparamindex> rptList) throws Exception {
        for (IcdConfigType configType : configTypeList) {
            List<IcdConfigTypeInst> instList = _instRepository.findByIcdConfigTypeId(configType.getId());
            List<IcdConfigTypeAtt> attList = _attRepository.findByIcdConfigTypeId(configType.getId());
            for (IcdConfigTypeInst inst : instList) {
                String param = configType.getIedName() + configType.getLdeviceInst() + "/" + configType.getLnClass() + inst.getInst();
                for (IcdConfigTypeAtt att : attList) {
                    if (att.containInst(inst.getInst())) {
                        String paramindexOld = param + "$" + att.getParam();
                        String paramindexNew = param + "." + att.getDoName() + "." + att.getLastName().replace("$", ".");
                        String paramindexT = param + "." + att.getDoName() + ".t";

                        Rptparamindex rpt = this.findRpt(rptList, paramindexOld);
                        if (rpt != null) {
                            this.collectAndSaveValue(paramindexNew, paramindexT, att.getFc(), rpt);
                        }
                    }
                }
            }
        }
    }

    private void collectAndSaveValue(String paramindexNew, String paramindexT, String fc, Rptparamindex rpt) throws Exception {
        String value = iecClient.getValue(paramindexNew, fc);
        String time = iecClient.getValue(paramindexT, fc);
        log.info("采集到" + fc + " " + paramindexNew + "=" + value + ", t=" + time);
        time = DateUtil.fromZoneUTCToLocal(time);
        _dataService.insertData(rpt.getTablename(), rpt.getEqmid(), time, rpt.getColname(), value);

        // 更新最新数据缓存
        updateLastData(rpt.getEqmid(), rpt.getColname(), value, time);

        // 更新关联关系缓存
        String key = paramindexNew + "_" + fc;
        value = rpt.getEqmid() + "," + rpt.getColname();
        StaticVariable.paramRelationMap.put(key, value);
        key = paramindexT + "_" + fc;
        value = rpt.getEqmid() + ",acquisitionTime";
        StaticVariable.paramRelationMap.put(key, value);
    }

    private Rptparamindex findRpt(List<Rptparamindex> rptList, String paramindex) {
        for (Rptparamindex rpt : rptList) {
            if (rpt.getParamindex().equals(paramindex)) {
                return rpt;
            }
        }
        return null;
    }

    private void doDownload(List<IedDlConfig> dlList) {
        for (IedDlConfig config : dlList) {
            this.downloadOne(config);
        }
    }

    private void downloadOne(IedDlConfig config) {
        if (config.getActive() == null || config.getActive().intValue() == Constants.FALSE) {
            return;
        }
        if (StringUtils.isBlank(config.getPath())) {
            return;
        }
        if (config.getDevId() == null) {
            return;
        }
        try {
            String localPath = folder + "/" + config.getDevId();
            List<FileInformation> fileList = iecClient.listFile(config.getPath());
            for (FileInformation file : fileList) {
                String filename = file.getFilename();
                if (!filename.endsWith("/")) {
                    if (StringUtils.isBlank(config.getSuffix()) ||
                            filename.toLowerCase().endsWith(config.getSuffix().toLowerCase())) {
                        IedDlRecord record = new IedDlRecord();
                        record.setConfigId(config.getId());
                        record.setFilename(filename);
                        boolean exist = _dlRecordService.exist(record);
                        if (!exist) {
                            String localFilePath = localPath + "/" + filename;
                            iecClient.getFile(config.getPath(), filename, _bizConfig.getDatapath() + localFilePath, config.getTodel());
                            record.setPath(_bizConfig.getDataNginxPath() + localFilePath);
                            record.setCreateTime(new Date());
                            _dlRecordService.add(record);
                        }
                    }
                }
            }
        } catch (Exception ex) {
            String err = "61850采集文件异常, ied=" + ied.getName() + ", ip=" + ied.getIp() + ", path=" + config.getPath();
            log.error(err, ex);
            String key = ied.getName() + ied.getIp() + ied.getPort() + config.getPath();
            if (!StaticVariable.doneWarnMap.containsKey(key)) {
                StaticVariable.doneWarnMap.put(key, "1");
                _webSocketServer.sendMessage(err, null);
            }
        }
    }

    public static void updateLastData(Integer eqmid, String colname, String value, String time) {
        HashMap<String, String> map = StaticVariable.sensorLastDataMap.get(eqmid);
        if (map == null) {
            map = new HashMap<>();
            StaticVariable.sensorLastDataMap.put(eqmid, map);
        }
        map.put(colname, value);
        if (time != null) {
            map.put("acquisitionTime", time);
        }
    }

    public static void updateLastData(BasicDataAttribute bda) {
        try {
            String ref = bda.getReference().toString();
            String key = ref + "_" + bda.getFc().toString();
            if (StaticVariable.paramRelationMap.containsKey(key)) {
                String value = StaticVariable.paramRelationMap.get(key);
                String[] str = value.split(",");
                Integer eqmid = Integer.parseInt(str[0]);
                String colname = str[1];
                value = bda.getValueString();
                if ("acquisitionTime".equals(colname)) {
                    value = DateUtil.fromZoneUTCToLocal(value);
                }
                updateLastData(eqmid, colname, value, null);
            }
        } catch (Exception ignore) {
        }
    }
}
