package com.xydl.cac.transform;

import com.alibaba.excel.util.StringUtils;
import com.xydl.cac.entity.*;
import com.xydl.cac.model.StaticVariable;
import com.xydl.cac.model.i2sync.Attr;
import com.xydl.cac.model.i2sync.Datanode;
import com.xydl.cac.model.i2sync.Monitordata;
import com.xydl.cac.model.i2sync.Request;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

@Component
@Slf4j
public class I2DataTransformer {

    public Request transform(NSensor sensor, I2syncRecord record, I2syncConfig config,
                             List<I2syncField> fieldList, List<Map<String, Object>> dataList) throws Exception {

        // 查到可用规则
        List<WarnRule> ruleList = new ArrayList<>();
        Iterator<Integer> it = StaticVariable.rule_Cache.keySet().iterator();
        while (it.hasNext()) {
            Integer key = it.next();
            WarnRule rule = StaticVariable.rule_Cache.get(key);
            if (rule.canDo() && rule.getSensorId().equals(sensor.getId())) {
                ruleList.add(rule);
            }
        }

        List<Datanode> nodeList = new ArrayList<>();
        for (Map<String, Object> map : dataList) {
            String date = (String) map.get("acquisitionTime");
            Datanode node = new Datanode();
            node.setSensorid(sensor.getSensorCode());
            node.setEquipmentid(sensor.getEquipmentId());
            node.setTimestamp(date);

            List<Attr> attrs = new ArrayList<>();
            Attr attr = new Attr();
            attr.setName("Phase");
            attr.setValue(sensor.getPhase());
            attr.setAlarm("FALSE");
            attrs.add(attr);
            for (I2syncField field : fieldList) {
                Object value = map.get(field.getFieldName());
                attr = new Attr();
                attr.setName(field.getDestFieldName());
                attr.setValue(String.valueOf(value));
                if (this.triggerRule(ruleList, field.getFieldName(), value)) {
                    attr.setAlarm("TRUE");
                } else {
                    attr.setAlarm("FALSE");
                }
                if (StringUtils.isBlank(node.getType())) {
                    node.setType(config.getTypeCode());
                }
                attrs.add(attr);
            }
            node.setAttr(attrs);

            nodeList.add(node);
            record.setLastDTime(DateUtil.parse(date));
        }

        Monitordata monitordata = new Monitordata();
        monitordata.setCacid(sensor.getSensorCode());
        monitordata.setDatanodenum(nodeList.size());
        monitordata.setDatanode(nodeList);
        Request request = new Request();
        request.setMonitordata(monitordata);
        return request;
    }

    private boolean triggerRule(List<WarnRule> ruleList, String fieldName, Object value) {
        for (WarnRule rule : ruleList) {
            if (rule.getTypePoint().getField().equalsIgnoreCase(fieldName)) {
                boolean r = rule.triggerRule(value);
                if (r) {
                    return true;
                }
            }
        }
        return false;
    }
}
